@@ -26,6 +26,7 @@
 #include <gsf/gsf-input-gzip.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-utils.h>
+#include <gsf/gsf-timestamp.h>
 
 #include <math.h>
 #include <string.h>
@@ -54,6 +55,21 @@ glade_enum_from_string (GType type, const char *string)
     return ret;
 }
 
+static char const *
+glade_string_from_enum (GType type, gint value)
+{
+    GEnumClass *eclass;
+    GEnumValue *ev;
+
+    eclass = g_type_class_ref(type);
+
+    ev = g_enum_get_value (eclass, value);
+
+    g_type_class_unref(eclass);
+
+    return ev ? ev->value_name : "0";
+}
+
 static guint
 glade_flags_from_string (GType type, const char *string)
 {
@@ -131,7 +147,56 @@ glade_flags_from_string (GType type, const char *string)
 
     return ret;
 }
+static gchar *
+glade_string_from_flags (GType type, guint flags)
+{
+    GFlagsClass *flags_class;
+    GString *string;
+    char *ret;
 
+    flags_class = g_type_class_ref (type);
+
+    string = g_string_new ("");
+
+    if (flags_class->n_values)
+      {
+	GFlagsValue *fval;
+      
+	for (fval = flags_class->values; fval->value_name; fval++)
+	  {
+	    /* We have to be careful as some flags include 0 values, e.g.
+	       BonoboDockItemBehavior uses 0 for BONOBO_DOCK_ITEM_BEH_NORMAL.
+	       If a 0 value is available, we only output it if the entire
+	       flags value is 0, otherwise we check if the bit-flag is set. */
+	    if ((fval->value == 0 && flags == 0)
+		|| (fval->value && (fval->value & flags) == fval->value))
+	      {
+		if (string->len)
+		  g_string_append_c (string, '|');
+		g_string_append (string, fval->value_name);
+	      }
+	  }
+      }
+
+    ret = string->str;
+    g_string_free (string, FALSE);
+
+    g_type_class_unref (flags_class);
+
+    return ret;
+}
+
+/**
+ * gsf_xml_gvalue_from_str :
+ * @res : Result value
+ * @t : Type of data
+ * @str : Value string
+ *
+ * Try to parse @str as a value of type @t into @res.
+ *
+ * Returns : True when parsing of @str as a value of type @t was succesfull;
+ * false otherwise.
+ */
 gboolean
 gsf_xml_gvalue_from_str (GValue *res, GType t, char const *str)
 {
@@ -181,6 +246,14 @@ gsf_xml_gvalue_from_str (GValue *res, GType t, char const *str)
 		break;
 
 	default:
+		if (GSF_TIMESTAMP_TYPE == t) {
+			GsfTimestamp ts;
+			if (gsf_timestamp_parse (str, &ts)) {
+				gsf_value_set_timestamp (res, &ts);
+				break;
+			}
+		}
+
 		return FALSE;
 	}
 	return TRUE;
@@ -221,6 +294,7 @@ static xmlParserCtxtPtr
 gsf_xml_parser_context_full (GsfInput *input, xmlSAXHandlerPtr sax, gpointer user)
 {
 	GsfInput *gzip;
+	xmlParserCtxtPtr res;
 
 	g_return_val_if_fail (GSF_IS_INPUT (input), NULL);
 
@@ -230,23 +304,30 @@ gsf_xml_parser_context_full (GsfInput *input, xmlSAXHandlerPtr sax, gpointer use
 	else
 		g_object_ref (G_OBJECT (input));
 
-	return xmlCreateIOParserCtxt (
+	res = xmlCreateIOParserCtxt (
 		sax, user,
 		(xmlInputReadCallback) gsf_libxml_read, 
 		(xmlInputCloseCallback) gsf_libxml_close,
 		input, XML_CHAR_ENCODING_NONE);
+
+	if (res)
+		res->replaceEntities = TRUE;
+	else
+		g_object_unref (input);
+
+	return res;
 }
 
 /**
  * gsf_xml_parser_context :
- * @input :
+ * @input : #GsfInput
  *
- * Create a libxml2 pull style parser context wrapper around a gsf input.
+ * Create a libxml2 pull style parser context wrapper around gsf input @input.
  * This signature will probably change to supply a SAX structure.
  *
- * NOTE : adds a reference to @input
- * NOTE : a simple wrapper around a cleaner implementation that will fold in
- *	 when we add other api changes.  Its not worth bumping just for this
+ * <note>This adds a reference to @input.</note>
+ * <note>A simple wrapper around a cleaner implementation that will fold in
+ * when we add other api changes.  Its not worth bumping just for this.</note>
  *
  * Returns : A parser context or NULL
  **/
@@ -258,11 +339,11 @@ gsf_xml_parser_context (GsfInput *input)
 
 /**
  * gsf_xml_output_buffer_new :
- * @output :
- * @encoding : optionally NULL.
+ * @output:
+ * @encoding: optionally NULL.
  *
- * NOTE : adds a reference to @output
- * NOTE : This is _not_ releated to GsfXMLOut
+ * <note>This adds a reference to @output.</note>
+ * <note>This is <emphasis>not</emphasis> releated to #GsfXMLOut.</note>
  */
 static xmlOutputBufferPtr
 gsf_xml_output_buffer_new (GsfOutput *output,
@@ -322,6 +403,7 @@ typedef struct {
 	GsfXMLInNode pub;
 	/* internal state */
 	GSList *groups;
+	GSList *extensions;
 } GsfXMLInNodeInternal;
 struct _GsfXMLInDoc {
 	GsfXMLInNodeInternal const *root_node;
@@ -333,14 +415,14 @@ struct _GsfXMLInDoc {
 typedef struct {
 	GsfXMLIn	pub;
 
-	GsfXMLInNS   const *default_ns;	/* optionally NULL */
-	GSList	 	   *ns_stack;
-	GHashTable	   *ns_prefixes;
-	GPtrArray	   *ns_by_id;
-	gint	  	    unknown_depth; /* handle recursive unknown tags */
-	gboolean	    from_unknown_handler;
+	GsfXMLInNS const *default_ns;	   /* optionally NULL */
+	GSList	 	 *ns_stack;
+	GHashTable	 *ns_prefixes;
+	GPtrArray	 *ns_by_id;
+	gint	  	  unknown_depth; /* handle recursive unknown tags */
+	gboolean	  from_unknown_handler;
 
-	GSList	 	   *extension_stack;
+	GSList	 	 *extension_stack; /* stack of GsfXMLInExtension */
 } GsfXMLInInternal;
 typedef struct {
 	char    *tag;
@@ -352,9 +434,9 @@ typedef struct {
 	GSList *elem;
 } GsfXMLInNodeGroup;
 typedef struct {
-	GsfXMLInDoc const *old_doc;
-	gpointer	   old_state;
-	GsfXMLInExtDtor    dtor;
+	GsfXMLInExtDtor	   dtor;
+	gpointer	   state;
+	GsfXMLInDoc const *doc;
 	gboolean	   from_unknown;
 } GsfXMLInExtension;
 
@@ -376,20 +458,32 @@ push_child (GsfXMLInInternal *state, GsfXMLInNode const *node, GsfXMLInNS const
 		(gpointer)state->pub.node);
 	state->ns_stack		= g_slist_prepend (state->ns_stack,
 		(gpointer)state->default_ns);
-	state->extension_stack	= g_slist_prepend (state->extension_stack, ext);
 	state->pub.node = node;
 	state->default_ns = default_ns;
-	if (node->start != NULL)
+
+	state->extension_stack	= g_slist_prepend (state->extension_stack, ext);
+	if (NULL != ext) {
+		GsfXMLInDoc const *old_doc = state->pub.doc;
+		state->pub.doc = ext->doc;
+		ext->doc = old_doc;
+		if (NULL != ext->state) {
+			gpointer old_state = state->pub.user_state;
+			state->pub.user_state = ext->state;
+			ext->state = old_state;
+		}
+	}
+	if (NULL != node->start)
 		node->start (&state->pub, attrs);
 }
 
 static gboolean
 lookup_child (GsfXMLInInternal *state, GsfXMLInNS const *default_ns,
-	      GsfXMLInNSInstance *inst,
-	      GSList *groups, xmlChar const *name, xmlChar const **attrs)
+	      GSList *groups, xmlChar const *name,
+	      xmlChar const **attrs, GsfXMLInExtension *ext)
 {
 	GsfXMLInNodeGroup  *group;
 	GsfXMLInNode	   *node;
+	GsfXMLInNSInstance *inst;
 	GSList *elem, *ptr;
 	char const *tmp;
 
@@ -413,7 +507,7 @@ lookup_child (GsfXMLInInternal *state, GsfXMLInNS const *default_ns,
 		for (elem = group->elem ; elem != NULL ; elem = elem->next) {
 			node = elem->data;
 			if (node->name == NULL || !strcmp (tmp, node->name)) {
-				push_child (state, node, default_ns, attrs, NULL);
+				push_child (state, node, default_ns, attrs, ext);
 				return TRUE;
 			}
 		}
@@ -461,7 +555,7 @@ gsf_xml_in_start_element (GsfXMLInInternal *state, xmlChar const *name, xmlChar
 					inst->tag    = g_strconcat (ns_ptr[0] + 6, ":", NULL);
 					inst->taglen = strlen (inst->tag);
 					inst->ref_count = 1;
-					g_hash_table_insert (state->ns_prefixes, inst->tag, inst);
+					g_hash_table_insert (state->ns_prefixes, g_strdup (ns_ptr[0] + 6), inst);
 
 					if (ns[i].ns_id >= state->ns_by_id->len)
 						g_ptr_array_set_size  (state->ns_by_id, ns[i].ns_id+1);
@@ -478,14 +572,22 @@ gsf_xml_in_start_element (GsfXMLInInternal *state, xmlChar const *name, xmlChar
 	}
 
 	node = (GsfXMLInNodeInternal const *) state->pub.node;
-	if (lookup_child (state, default_ns, inst, node->groups, name, attrs))
+	if (lookup_child (state, default_ns, node->groups, name, attrs, NULL))
 		return;
 
 	/* useful for <Data><b><i><u></u></i></b></Data> where all of the markup can nest */
 	ptr = state->pub.node_stack;
 	for (; ptr != NULL && node->pub.share_children_with_parent; ptr = ptr->next) {
 		node = ptr->data;
-		if (lookup_child (state, default_ns, inst, node->groups, name, attrs))
+		if (lookup_child (state, default_ns, node->groups, name, attrs, NULL))
+			return;
+	}
+
+	/* Check for extensions */
+	for (ptr = node->extensions; ptr != NULL ; ptr = ptr->next) {
+		GsfXMLInExtension *ext = ptr->data;
+		if (lookup_child (state, default_ns,
+			ext->doc->root_node->groups, name, attrs, ext))
 			return;
 	}
 
@@ -515,46 +617,63 @@ gsf_xml_in_start_element (GsfXMLInInternal *state, xmlChar const *name, xmlChar
 	state->pub.node_stack = g_slist_reverse (state->pub.node_stack);
 }
 
+static void
+gsf_xml_in_ext_free (GsfXMLInInternal *state, GsfXMLInExtension *ext)
+{
+	if (ext->dtor)
+		(ext->dtor) (&state->pub, ext->state);
+	g_free (ext);
+}
+
 static void
 gsf_xml_in_end_element (GsfXMLInInternal *state,
 			G_GNUC_UNUSED xmlChar const *name)
 {
-	GsfXMLInExtension *ext;
-	gboolean was_overlay;
+	GsfXMLInNodeInternal	*node;
+	GsfXMLInExtension	*ext;
+	GSList *ptr;
 
 	if (state->unknown_depth > 0) {
 		state->unknown_depth--;
 		return;
 	}
 
-	do {
-		g_return_if_fail (state->pub.node_stack != NULL);
-		g_return_if_fail (state->ns_stack != NULL);
-		g_return_if_fail (state->extension_stack != NULL);
+	g_return_if_fail (state->pub.node	!= NULL);
+	g_return_if_fail (state->pub.node_stack != NULL);
+	g_return_if_fail (state->ns_stack != NULL);
 
-		if (state->pub.node->end)
-			state->pub.node->end (&state->pub, NULL);
-		if (state->pub.node->has_content == GSF_XML_CONTENT)
-			g_string_truncate (state->pub.content, 0);
+	node = (GsfXMLInNodeInternal *) state->pub.node;
+	if (node->pub.end)
+		node->pub.end (&state->pub, NULL);
+	if (node->pub.has_content == GSF_XML_CONTENT)
+		g_string_truncate (state->pub.content, 0);
 
-		/* pop the state stack */
-		ext = state->extension_stack->data;
-		state->extension_stack	= g_slist_remove (state->extension_stack, ext);
-		state->pub.node		= state->pub.node_stack->data;
-		state->pub.node_stack	= g_slist_remove (state->pub.node_stack, state->pub.node);
-		state->default_ns	= state->ns_stack->data;
-		state->ns_stack		= g_slist_remove (state->ns_stack, state->default_ns);
+	/* Free any potential extensions associated with the current node */
+	for (ptr = node->extensions; ptr != NULL ; ptr = ptr->next)
+		gsf_xml_in_ext_free (state, ptr->data);
+	g_slist_free (node->extensions);
+	node->extensions = NULL;
 
-		if (NULL != ext) {
-			was_overlay = !ext->from_unknown;
-			if (ext->dtor)
-				(ext->dtor) (&state->pub, ext->old_state);
-			state->pub.user_state = ext->old_state;
-			state->pub.doc = ext->old_doc;
-			g_free (ext);
-		} else
-			was_overlay = FALSE;
-	} while (was_overlay);
+	/* pop the state stack */
+	ext = state->extension_stack->data;
+	state->extension_stack	= g_slist_remove (state->extension_stack, ext);
+	state->pub.node		= state->pub.node_stack->data;
+	state->pub.node_stack	= g_slist_remove (state->pub.node_stack, state->pub.node);
+	state->default_ns	= state->ns_stack->data;
+	state->ns_stack		= g_slist_remove (state->ns_stack, state->default_ns);
+
+	if (NULL != ext) {
+		GsfXMLInDoc const *ext_doc = state->pub.doc;
+		state->pub.doc = ext->doc;
+		ext->doc = ext_doc;
+		if (NULL != ext->state) {
+			gpointer ext_state = state->pub.user_state;
+			state->pub.user_state = ext->state;
+			ext->state = ext_state;
+		}
+		if (ext->from_unknown)
+			gsf_xml_in_ext_free (state, ext);
+	}
 }
 
 static void
@@ -570,6 +689,13 @@ gsf_xml_in_get_entity (G_GNUC_UNUSED GsfXMLInInternal *state, xmlChar const *nam
 	return xmlGetPredefinedEntity (name);
 }
 
+static void
+gsf_free_xmlinnsinstance (GsfXMLInNSInstance *inst)
+{
+	g_free (inst->tag);
+	g_free (inst);
+}
+
 static void
 gsf_xml_in_start_document (GsfXMLInInternal *state)
 {
@@ -581,7 +707,9 @@ gsf_xml_in_start_document (GsfXMLInInternal *state)
 	state->default_ns	= NULL;
 	state->ns_by_id		= g_ptr_array_new ();
 	state->ns_prefixes	= g_hash_table_new_full (
-		g_str_hash, g_str_equal, NULL, g_free);
+		g_str_hash, g_str_equal,
+		g_free, (GDestroyNotify) gsf_free_xmlinnsinstance);
+	state->from_unknown_handler = FALSE;
 }
 
 static void
@@ -675,6 +803,10 @@ gsf_xml_in_node_internal_free (GsfXMLInNodeInternal *node)
 	GSList *ptr;
 	GsfXMLInNodeGroup *group;
 
+	if (NULL != node->extensions) {
+		g_warning ("leaking extensions");
+	}
+
 	for (ptr = node->groups; ptr != NULL ; ptr = ptr->next) {
 		group = ptr->data;
 		g_slist_free (group->elem);
@@ -838,15 +970,14 @@ gsf_xml_in_push_state (GsfXMLIn *xin, GsfXMLInDoc const *doc,
 	g_return_if_fail (doc->root_node != NULL);
 
 	ext = g_new (GsfXMLInExtension, 1);
-	ext->old_doc	= xin->doc;
-	ext->old_state	= xin->user_state;
-	ext->dtor	= dtor;
-	ext->from_unknown = state->from_unknown_handler;
-
-	xin->doc = doc;
-	xin->user_state = new_state;
-
-	push_child (state, &doc->root_node->pub, NULL, attrs, ext);
+	ext->doc	  = doc;
+	ext->state	  = new_state;
+	ext->dtor	  = dtor;
+	if (!(ext->from_unknown = state->from_unknown_handler)) {
+		GsfXMLInNodeInternal *node = (GsfXMLInNodeInternal *) xin->node;
+		node->extensions = g_slist_prepend (node->extensions, ext);
+	} else
+		push_child (state, &doc->root_node->pub, NULL, attrs, ext);
 }
 
 /**
@@ -989,6 +1120,14 @@ GSF_CLASS (GsfXMLOut, gsf_xml_out,
 	   gsf_xml_out_class_init, gsf_xml_out_init,
 	   G_TYPE_OBJECT)
 
+/**
+ * gsf_xml_out_new :
+ * @output : #GsfOutput
+ *
+ * Create an XML output stream.
+ *
+ * Returns: #GsfXMLOut
+ */
 GsfXMLOut *
 gsf_xml_out_new (GsfOutput *output)
 {
@@ -1002,8 +1141,8 @@ gsf_xml_out_new (GsfOutput *output)
 /**
  * gsf_xml_out_set_doc_type :
  * @xml : #GsfXMLOut
- * @type :
- * 
+ * @type : the document type declaration
+ *
  * Store some optional some &lt;!DOCTYPE .. &gt; content
  **/
 void
@@ -1030,9 +1169,11 @@ gsf_xml_out_indent (GsfXMLOut *xml)
 }
 
 /**
- * gsf_xml_out_start_elem :
- * @xml :
- * @id  :
+ * gsf_xml_out_start_element :
+ * @xml : #GsfXMLOut
+ * @id  : Element name
+ *
+ * Output a start element @id, if necessary preceeded by an XML declaration.
  */
 void
 gsf_xml_out_start_element (GsfXMLOut *xml, char const *id)
@@ -1061,8 +1202,12 @@ gsf_xml_out_start_element (GsfXMLOut *xml, char const *id)
 }
 
 /**
- * gsf_xml_out_end_elem :
- * @xml :
+ * gsf_xml_out_end_element :
+ * @xml : #GsfXMLOut
+ *
+ * Closes/ends an XML element.
+ *
+ * Returns: the element that has been closed.
  */
 char const *
 gsf_xml_out_end_element (GsfXMLOut *xml)
@@ -1092,11 +1237,11 @@ gsf_xml_out_end_element (GsfXMLOut *xml)
 
 /**
  * gsf_xml_out_simple_element :
- * @xml :
- * @id  :
- * @content :
+ * @xml : #GsfXMLOut
+ * @id  : Element name
+ * @content : Content of the element
  *
- * A convenience routine
+ * Convenience routine to output a simple @id element with content @content.
  **/
 void
 gsf_xml_out_simple_element (GsfXMLOut *xml, char const *id,
@@ -1110,11 +1255,11 @@ gsf_xml_out_simple_element (GsfXMLOut *xml, char const *id,
 
 /**
  * gsf_xml_out_simple_int_element :
- * @xml :
- * @id  :
- * @val :
+ * @xml : #GsfXMLOut
+ * @id  : Element name
+ * @val : Element value
  *
- * A convenience routine
+ * Convenience routine to output an element @id with integer value @val.
  **/
 void
 gsf_xml_out_simple_int_element (GsfXMLOut *xml, char const *id, int val)
@@ -1126,12 +1271,13 @@ gsf_xml_out_simple_int_element (GsfXMLOut *xml, char const *id, int val)
 
 /**
  * gsf_xml_out_simple_float_element :
- * @xml :
- * @id  :
- * @val :
- * @precision :
+ * @xml : #GsfXMLOut
+ * @id  : Element name
+ * @val : Element value
+ * @precision : the number of significant digits to use, -1 meaning "enough".
  *
- * A convenience routine
+ * Convenience routine to output an element @id with float value @val using
+ * @precision significant digits.
  **/
 void
 gsf_xml_out_simple_float_element (GsfXMLOut *xml, char const *id,
@@ -1153,7 +1299,7 @@ close_tag_if_neccessary (GsfXMLOut* xml)
 
 /**
  * gsf_xml_out_add_cstr_unchecked :
- * @xml :
+ * @xml : #GsfXMLOut
  * @id : optionally NULL for content
  * @val_utf8 : a utf8 encoded string to export
  *
@@ -1180,7 +1326,7 @@ gsf_xml_out_add_cstr_unchecked (GsfXMLOut *xml, char const *id,
 
 /**
  * gsf_xml_out_add_cstr :
- * @xml :
+ * @xml : #GsfXMLOut
  * @id : optionally NULL for content
  * @val_utf8 : a utf8 encoded string
  *
@@ -1250,7 +1396,7 @@ gsf_xml_out_add_cstr (GsfXMLOut *xml, char const *id,
 
 /**
  * gsf_xml_out_add_bool :
- * @xml :
+ * @xml : #GsfXMLOut
  * @id  : optionally NULL for content
  * @val : a boolean
  *
@@ -1267,7 +1413,7 @@ gsf_xml_out_add_bool (GsfXMLOut *xml, char const *id,
 
 /**
  * gsf_xml_out_add_int :
- * @xml :
+ * @xml : #GsfXMLOut
  * @id  : optionally NULL for content
  * @val : the value
  *
@@ -1284,7 +1430,7 @@ gsf_xml_out_add_int (GsfXMLOut *xml, char const *id,
 
 /**
  * gsf_xml_out_add_uint :
- * @xml :
+ * @xml : #GsfXMLOut
  * @id  : optionally NULL for content
  * @val : the value
  *
@@ -1302,7 +1448,7 @@ gsf_xml_out_add_uint (GsfXMLOut *xml, char const *id,
 
 /**
  * gsf_xml_out_add_float :
- * @xml :
+ * @xml : #GsfXMLOut
  * @id  : optionally NULL for content
  * @val : the value
  * @precision : the number of significant digits to use, -1 meaning "enough".
@@ -1328,11 +1474,11 @@ gsf_xml_out_add_float (GsfXMLOut *xml, char const *id,
 
 /**
  * gsf_xml_out_add_color :
- * @xml :
+ * @xml : #GsfXMLOut
  * @id  : optionally NULL for content
- * @r :
- * @g :
- * @b :
+ * @r : Red value
+ * @g : Green value
+ * @b : Blue value
  *
  * dump Color @r.@g.@b to an attribute named @id or as the nodes content
  **/
@@ -1345,6 +1491,15 @@ gsf_xml_out_add_color (GsfXMLOut *xml, char const *id,
 	gsf_xml_out_add_cstr_unchecked (xml, id, buf);
 }
 
+/**
+ * gsf_xml_out_add_enum :
+ * @xml : #GsfXMLOut
+ * @id  : optionally NULL for content
+ * @etype : #GType
+ * @val : enum element number
+ *
+ * Output the name of value @val of enumeration type @etype.
+ **/
 void
 gsf_xml_out_add_enum (GsfXMLOut *xml, char const *id, GType etype, gint val)
 {
@@ -1359,11 +1514,89 @@ gsf_xml_out_add_enum (GsfXMLOut *xml, char const *id, GType etype, gint val)
 }
 
 /**
- * gsf_xml_out_add_base64 :
- * @xml :
+ * gsf_xml_out_add_gvalue :
+ * @xml : #GsfXMLOut
  * @id  : optionally NULL for content
- * @data :
- * @len :
+ * @val : #GValue
+ *
+ * Output the value of @val as a string.  Does NOT store any type information
+ * with the string, just thevalue.
+ **/
+void
+gsf_xml_out_add_gvalue (GsfXMLOut *xml, char const *id, GValue const *val)
+{
+	GType t;
+	g_return_if_fail (xml != NULL);
+	g_return_if_fail (val != NULL);
+
+	t = G_VALUE_TYPE (val);
+	switch (t) {
+	case G_TYPE_CHAR: {
+		char c[2] = { 0, 0 };
+		c[0] = g_value_get_char (val);
+		gsf_xml_out_add_cstr (xml, id, c);
+		break;
+	}
+
+	case G_TYPE_UCHAR: {
+		unsigned char c[2] = { 0, 0 };
+		c[0] = g_value_get_uchar (val);
+		gsf_xml_out_add_cstr (xml, id, c);
+		break;
+	}
+
+	case G_TYPE_BOOLEAN:
+		gsf_xml_out_add_cstr (xml, id,
+			g_value_get_boolean (val) ? "t" : "f");
+		break;
+	case G_TYPE_INT:
+		gsf_xml_out_add_int (xml, id, g_value_get_int (val));
+		break;
+	case G_TYPE_UINT:
+		gsf_xml_out_add_uint (xml, id, g_value_get_uint (val));
+		break;
+	case G_TYPE_LONG:
+		gsf_xml_out_add_uint (xml, id, g_value_get_long (val));
+		break;
+	case G_TYPE_ULONG:
+		gsf_xml_out_add_uint (xml, id, g_value_get_ulong (val));
+		break; 
+	case G_TYPE_ENUM:
+		gsf_xml_out_add_cstr (xml, id,
+			glade_string_from_enum (t, g_value_get_enum (val)));
+		break;
+	case G_TYPE_FLAGS:
+		gsf_xml_out_add_cstr (xml, id,
+			glade_string_from_flags (t, g_value_get_flags (val)));
+		break;
+	case G_TYPE_FLOAT:
+		gsf_xml_out_add_float (xml, id, g_value_get_float (val), -1);
+		break;
+	case G_TYPE_DOUBLE:
+		gsf_xml_out_add_float (xml, id, g_value_get_double (val), -1);
+		break;
+	case G_TYPE_STRING:
+		gsf_xml_out_add_cstr (xml, id, g_value_get_string (val));
+		break;
+
+	default:
+		if (GSF_TIMESTAMP_TYPE == t) {
+			char *str = gsf_timestamp_as_string (
+				(GsfTimestamp const *)g_value_get_boxed (val));
+			gsf_xml_out_add_cstr (xml, id, str);
+			break;
+		}
+	}
+
+	/* FIXME FIXME FIXME Add some error checking */
+}
+
+/**
+ * gsf_xml_out_add_base64 :
+ * @xml : #GsfXMLOut
+ * @id  : optionally NULL for content
+ * @data : Data to be written
+ * @len : Length of data
  *
  * dump @len bytes in @data into the content of node @id using base64
  **/
