@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -49,7 +49,7 @@ test (int argc, char *argv[])
 	GsfOutfile *outfile;
 	GsfOutput  *small;
 	GsfOutput  *large;
-	GError   *err;
+	GError   *err = NULL;
 	int i;
 
 	if (argc != 2) {
